from .__version__ import (
    __title__,
    __url__,
    __version__,
    __author__,
    __author_email__,
    __license__,
    __copyright__,
)

from .auto_switch_providers import AutoSwitchProviders
from .http_service import HttpService
from .utils.helpers import merge_child, nested_dict_get
