__title__ = "auto-switch-providers"
__description__ = "Auto Switch Providers"
__url__ = "https://github.com/fuongz/auto-switch-providers"
__version__ = "0.0.5"
__author__ = "fuongz"
__author_email__ = "hi@phuongphung.com"
__license__ = "MIT"
__copyright__ = "Copyright fuongz"
